import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import Check
import Core
import I18n

import "../widgets"

Item {
    id: root
    clip: true

    property bool edit
    property color backgroundColor
    property bool forbidReduce: false

    property var checkModel
    property alias model: list.model
    property alias delegate: list.delegate
    property alias interactive: list.interactive

    function scrollToEnd() { list.positionViewAtEnd() }

    // Создает эффект прозрачности для элементов ListView, которые находятся на границах (сверху и снизу)
    function getItemOpasity(item) {
        const crossingTop = item.y < list.contentY
        const crossingBottom = item.y + item.height > list.contentY + list.height
        const fullVisible = !crossingTop && !crossingBottom
        return fullVisible ? 1 : Math.pow(1 - (crossingTop ? (list.contentY - item.y) / item.height : ((item.y + item.height) - (list.contentY + list.height)) / item.height), 2)
    }

    // Для выделения кол-ва позиции при нажатии +, - или занчения
    property int lastPositionInteracted: -1
    Timer {
        id: qtyTimer
        interval: 2000
        repeat: false
        triggeredOnStart: false
        onTriggered: lastPositionInteracted = -1
    }

    RowLayout {
        spacing: 10
        anchors.fill: parent

        component QuantityChange: Item {
            id: qtyChange
            property int posId
            property var qty
            property string units
            property bool edit: true
            property bool reduce: true
            property bool increase: true
            property bool groupItem: false

            RowLayout {
                anchors.fill: parent
                spacing: Style.parameter("geometry.positionList.positionDelegate.quantityChangeSpacing", 20)

                ActionButton {
                    visible: root.edit && !groupItem
                    Style.class: "icon"
                    enabled: !forbidReduce
                    contentItem: SvgImage {
                        antialiasing: Style.parameter("geometry.positionList.positionDelegate.iconsAntialiasing", false)
                        visible: !forbidReduce || !qty.fractional
                        source: {
                            if (!enabled)
                                return Style.parameter("icon.position.qtyMinusDisabled")
                            if (qty.fractional || qty.str === "1")
                                return Style.parameter("icon.position.qtyDelete")
                            return Style.parameter("icon.position.qtyMinus")
                        }
                    }
                    padding: Style.parameter("geometry.positionList.positionDelegate.quantityChangeButtonsPadding", 15)

                    Layout.preferredWidth: Style.parameter("geometry.positionList.positionDelegate.quantityChangeButtonsSize", 60)
                    Layout.preferredHeight: Style.parameter("geometry.positionList.positionDelegate.quantityChangeButtonsSize", 60)

                    onClicked: () => {
                        lastPositionInteracted = qtyChange.posId
                        Check.selectItemByPosId(qtyChange.posId)
                        PluginManager.async("CHECK_POSITIONQUANTITYMINUS")
                    }
                }

                Space {
                    visible: qtyChange.groupItem
                    width: 30
                }

                SGroupBox {
                    Style.class: "quantityBox"
                    padding: 0
                    enabled: qty && !qty.fractional && qtyChange.edit
                    down: posId === lastPositionInteracted
                    clip: true

                    Layout.preferredWidth: Style.parameter("geometry.positionList.positionDelegate.quantityChangeBoxWidth", 90)
                    Layout.preferredHeight: Style.parameter("geometry.positionList.positionDelegate.quantityChangeBoxHeight", 60)

                    onDownChanged: if (down) qtyTimer.restart()

                    contentItem: SLabel {
                        Style.class: qtyChange.groupItem ? "groupItemQuantity" : "quantity"
                        text: qty.str + " " + units
                        horizontalAlignment: Qt.AlignHCenter
                        verticalAlignment: Qt.AlignVCenter
                    }

                    MouseArea {
                        anchors.fill: parent

                        onClicked: () => {
                            lastPositionInteracted = qtyChange.posId
                            Check.selectItemByPosId(qtyChange.posId)
                            PluginManager.async("CHECK_CHANGEQUANTITY", { forbidReduce: forbidReduce })
                        }
                    }
                }

                Space {
                    visible: !qtyChange.increase
                    width: 75
                }

                ActionButton {
                    visible: root.edit && !groupItem && qtyChange.increase
                    Style.class: "icon"
                    enabled: qty && !qty.fractional && qtyChange.increase
                    contentItem: SvgImage {
                        visible: qty && !qty.fractional
                        antialiasing: Style.parameter("geometry.positionList.positionDelegate.iconsAntialiasing", false)
                        source: Style.parameter("icon.position.qtyPlus")
                    }
                    padding: Style.parameter("geometry.positionList.positionDelegate.quantityChangeButtonsPadding", 15)

                    Layout.preferredWidth: Style.parameter("geometry.positionList.positionDelegate.quantityChangeButtonsSize", 60)
                    Layout.preferredHeight: Style.parameter("geometry.positionList.positionDelegate.quantityChangeButtonsSize", 60)

                    onClicked: () => {
                        lastPositionInteracted = qtyChange.posId
                        Check.selectItemByPosId(qtyChange.posId)
                        PluginManager.async("CHECK_POSITIONQUANTITYPLUS")
                    }
                }
            }
        }

        component Separator: Rectangle {
            id: separator
            color: Style.color("graySoft")
            height: 1
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.bottom: parent.bottom
        }

        component Position: Item {
            id: position
            property int posId
            property string name
            property var quantity
            property var groupItem
            property string units
            property string groupId
            property var sum
            property var sumNoDisc
            property var disc
            property int age
            property int type

            RowLayout {
                anchors.fill: parent
                spacing: groupItem ? 8 : Style.parameter("geometry.positionList.positionDelegate.spacing", 40)

                Space {
                    width: 5
                    visible: position.groupItem
                }

                SvgImage {
                    visible: position.groupItem
                    source: Style.parameter("icon.group.item")
                    Layout.alignment: Qt.AlignCenter
                }

                SLabel {
                    Style.class: groupItem ? "groupItemPositionName" : "positionName"
                    text: name
                    Tr {
                        code: { return "couponName" }
                        args: [name]
                        enabled: type === CheckTypes.ItemType.Coupon
                    }
                    elide: SLabel.ElideRight
                    wrapMode: Text.WordWrap
                    maximumLineCount: 2
                    Layout.fillWidth: true
                }

                SGroupBox {
                    Style.class: "positionAgeVerify"
                    visible: age > 0 && !groupItem
                    topPadding: 7
                    rightPadding: 5
                    bottomPadding: 7
                    leftPadding: 5

                    contentItem: SLabel {
                        Style.class: "positionAgeVerify"
                        text: "18+"
                    }
                }

                Space {
                    visible: groupItem
                    width: 150

                }

                QuantityChange {
                    reduce: !position.groupItem
                    edit: !position.groupItem
                    increase: !position.groupItem
                    posId: position.posId
                    qty: position.quantity
                    units: position.units
                    groupItem: position.groupItem

                    Layout.preferredWidth: Style.parameter("geometry.positionList.positionDelegate.quantityWidth", 220)
                    Layout.preferredHeight: Style.parameter("geometry.positionList.delegateQuantityHeight", 60)
                }

                ColumnLayout {
                    visible: type != CheckTypes.ItemType.Coupon && !position.groupItem
                    Layout.minimumWidth: Style.parameter("geometry.positionList.positionDelegate.sumMinWidth", 130)

                    SMoney {
                        Style.class: "positionSumNoDisc"
                        visible: sumNoDisc !== sum
                        discount: true
                        backgroundColor: root.backgroundColor
                        money: sumNoDisc

                        Layout.alignment: Qt.AlignRight
                    }

                    SMoney {
                        Style.class: "positionSum"
                        money: sum

                        Layout.alignment: Qt.AlignRight
                    }
                }

                Space {
                    visible: position.groupItem
                    Layout.minimumWidth: Style.parameter("geometry.positionList.positionDelegate.sumMinWidth", 130)
                }
            }
        }

        component Section : Item {
            id: sectionHeading
            visible:  section.length > 0

            RowLayout {
                anchors.fill: parent
                spacing: 40

                RowLayout {
                    spacing: 10
                    SvgImage {
                        source: Style.parameter("icon.group.header")
                        Layout.preferredWidth: 35
                        Layout.preferredHeight: 35
                        Layout.alignment: Qt.AlignCenter

                    }

                    SLabel {
                        Style.class: "positionName"
                        text: checkModel.groupName(section)
                        elide: SLabel.ElideRight
                        wrapMode: Text.WordWrap
                        maximumLineCount: 2
                        Layout.fillWidth: true
                    }
                }

                SGroupBox {
                    Style.class: "positionAgeVerify"
                    visible: checkModel.groupAge(section) > 0
                    topPadding: 7
                    rightPadding: 5
                    bottomPadding: 7
                    leftPadding: 5

                    contentItem: SLabel {
                        Style.class: "positionAgeVerify"
                        text: "18+"
                    }
                }

                QuantityChange {
                    id: groupQty
                    qty: Quantity.fromCount(checkModel.groupQty(section))
                    units: "шт."
                    posId: checkModel.groupPosId(section)
                    increase: checkModel.groupQuantityEdit(section)
                    edit: checkModel.groupQuantityEdit(section)
                    groupItem: false

                    Layout.preferredWidth: Style.parameter("geometry.positionList.positionDelegate.quantityWidth", 220)
                    Layout.preferredHeight: Style.parameter("geometry.positionList.delegateQuantityHeight", 60)
                }

                ColumnLayout {
                    id: groupSum
                    Layout.minimumWidth: 130


                    SMoney {
                        Style.class: "positionSumNoDisc"
                        visible: checkModel.groupSumNoDisc(section) !== checkModel.groupSum(section)
                        discount: true
                        backgroundColor: root.backgroundColor
                        money: checkModel.groupSumNoDisc(section)
                        Layout.alignment: Qt.AlignRight
                    }

                    SMoney {
                        Style.class: "positionSum"
                        money: checkModel.groupSum(section)
                        Layout.alignment: Qt.AlignRight
                    }
                }
            }
        }

        ListView {
            id: list

            property var newPos: []

            delegate: Control {
                function getItemHeight(groupItem, lastOfGroup) {
                    if (!groupItem)
                        return Style.parameter("geometry.positionList.positionDelegate.height", 100)
                    return Style.parameter("geometry.positionList.groupPositionDelegate.height", 66)
                }

                id: itemContainer
                width: list.width
                leftPadding: 20
                rightPadding: 20
                height: getItemHeight(model.groupItem, checkModel.lastOfGroup(model.posId))
                opacity: getItemOpasity(this)

                Separator {
                    visible: !model.groupItem
                }

                contentItem: Item {
                    Position  {
                        anchors.fill: parent
                        posId: model.posId
                        name: model.name
                        groupItem: model.groupItem
                        quantity: model.quantity
                        units: model.units
                        sum: model.sum
                        disc: model.disc
                        sumNoDisc: model.sumNoDisc
                        age: model.age
                        groupId: model.groupId
                        type: model.type
                    }
                }

                background: Rectangle {
                    color: Style.color("green")
                    radius: 20
                    opacity: 0

                    NumberAnimation on opacity {
                        easing.type: Easing.InExpo
                        running: list.newPos.indexOf(model.row) !== -1
                        from: 0.25; to: 0
                        duration: Style.animationDuration("positionList.highlight", 1000)

                        onFinished: () => {
                                        const idx = list.newPos.indexOf(model.row)
                                        if (idx !== -1) list.newPos.splice(idx, 1)
                                    }
                    }
                }

                Separator {
                    visible: checkModel.lastOfGroup(model.posId)
                }
            }

            Connections {
                target: root.model

                function onRowsInserted(_parent, first, last) {
                    for (let i = first; i <= last; ++i)
                        list.newPos.push(i)
                    Qt.callLater(list.positionViewAtIndex, last, ListView.Contain)
                }
            }

            ScrollBar.vertical: scrollBar

            onHeightChanged: () => {
                                 if (list.newPos.length > 0)
                                 list.positionViewAtIndex(list.newPos[list.newPos.length - 1], ListView.Contain)
                             }

            section {
                property: "groupId"
                criteria: ViewSection.FullString
                delegate: Control {
                    height: contentItem.visible ? Style.parameter("geometry.group.delegateHeight", 86) : 0
                    visible: contentItem.visible
                    width: list.width
                    leftPadding: 20
                    rightPadding: 20
                    opacity: getItemOpasity(this)
                    contentItem: Section{}
                }
            }

            Layout.fillWidth: true
            Layout.fillHeight: true
        }

        ScrollBar {
            id: scrollBar
            policy: ScrollBar.AlwaysOn
            padding: 0

            background: Rectangle {
                color: Style.color("graySoft")
                radius: width / 2
            }

            contentItem: Rectangle {
                color: Style.color("green")
                radius: width / 2
            }

            Behavior on Layout.preferredWidth {
                NumberAnimation { easing.type: Easing.InOutQuad; duration: Style.animationDuration("positionList.scrollBar", 250) }
            }

            function getWidth() {
                if (!list.contentHeight || !list.height)
                    return 0
                return list.contentHeight > list.height && size < 0.99 ? 10 : 0
            }
            opacity: size < 0.99 ? 1 : 0
            Layout.preferredWidth: getWidth()
            Layout.fillHeight: true
        }
    }
}
